"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.start = start;
var _nodeReadline = _interopRequireDefault(require("node:readline"));
var _nodeUrl = require("node:url");
var _execa = _interopRequireDefault(require("execa"));
var _env = require("../env");
var _logging = require("../logging");
var _Compiler = require("../webpack/Compiler");
var _getConfigFilePath = require("./utils/getConfigFilePath");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/**
 * Start command for React Native CLI.
 * It runs `@callstack/repack-dev-server` to provide Development Server functionality to React Native apps
 * in development mode.
 *
 * @param _ Original, non-parsed arguments that were provided when running this command.
 * @param config React Native CLI configuration object.
 * @param args Parsed command line arguments.
 *
 * @internal
 * @category CLI command
 */
async function start(_, cliConfig, args) {
  const configPath = (0, _getConfigFilePath.getConfigFilePath)(cliConfig.root, args.webpackConfig);
  const {
    platforms: platformsArg,
    reversePort: reversePortArg,
    ...restArgs
  } = args;
  const cliOptions = {
    config: {
      root: cliConfig.root,
      reactNativePath: cliConfig.reactNativePath,
      webpackConfigPath: configPath
    },
    command: 'start',
    arguments: {
      start: {
        platforms: platformsArg ?? _env.DEFAULT_PLATFORMS,
        ...restArgs
      }
    }
  };
  const reversePort = reversePortArg ?? process.argv.includes('--reverse-port');
  const isSilent = args.silent;
  const isVerbose = isSilent ? false : args.verbose ?? process.argv.includes('--verbose');
  const reporter = (0, _logging.composeReporters)([new _logging.ConsoleReporter({
    asJson: args.json,
    level: isSilent ? 'silent' : isVerbose ? 'verbose' : 'normal'
  }), args.logFile ? new _logging.FileReporter({
    filename: args.logFile
  }) : undefined].filter(Boolean));
  const compiler = new _Compiler.Compiler(cliOptions, reporter);
  const {
    createServer
  } = await import('@callstack/repack-dev-server');
  const {
    start,
    stop
  } = await createServer({
    options: {
      rootDir: cliOptions.config.root,
      host: args.host || _env.DEFAULT_HOSTNAME,
      port: args.port ?? _env.DEFAULT_PORT,
      https: args.https ? {
        cert: args.cert,
        key: args.key
      } : undefined,
      logRequests: isVerbose || (args.logRequests ?? process.argv.includes('--log-requests'))
    },
    experiments: {
      experimentalDebugger: args.experimentalDebugger
    },
    delegate: async ctx => {
      if (args.interactive) {
        bindKeypressInput(ctx);
      }
      if (reversePort && args.port) {
        void runAdbReverse(ctx, args.port);
      }
      await compiler.init(ctx);
      return {
        compiler: {
          getAsset: async (filename, platform) => (await compiler.getAsset(filename, platform)).data,
          getMimeType: filename => compiler.getMimeType(filename),
          inferPlatform: uri => {
            const url = new _nodeUrl.URL(uri, 'protocol://domain');
            if (!url.searchParams.get('platform')) {
              const [, platform] = /^\/(.+)\/.+$/.exec(url.pathname) ?? [];
              return platform;
            }
            return undefined;
          }
        },
        symbolicator: {
          getSource: fileUrl => {
            const {
              filename,
              platform
            } = parseFileUrl(fileUrl);
            return compiler.getSource(filename, platform);
          },
          getSourceMap: fileUrl => {
            // TODO Align this with output.hotModuleUpdateChunkFilename
            if (fileUrl.endsWith('.hot-update.js')) {
              const {
                pathname
              } = new _nodeUrl.URL(fileUrl);
              const [platform, filename] = pathname.split('/').filter(Boolean);
              return compiler.getSourceMap(filename, platform);
            }
            const {
              filename,
              platform
            } = parseFileUrl(fileUrl);
            if (!platform) {
              throw new Error('Cannot infer platform for file URL');
            }
            return compiler.getSourceMap(filename, platform);
          },
          shouldIncludeFrame: frame => {
            // If the frame points to internal bootstrap/module system logic, skip the code frame.
            return !/webpack[/\\]runtime[/\\].+\s/.test(frame.file);
          }
        },
        hmr: {
          getUriPath: () => '/__hmr',
          onClientConnected: (platform, clientId) => {
            ctx.broadcastToHmrClients({
              action: 'sync',
              body: compiler.getHmrBody(platform)
            }, platform, [clientId]);
          }
        },
        messages: {
          getHello: () => 'React Native packager is running',
          getStatus: () => 'packager-status:running'
        },
        logger: {
          onMessage: log => {
            const logEntry = (0, _logging.makeLogEntryFromFastifyLog)(log);
            logEntry.issuer = 'DevServer';
            reporter.process(logEntry);
          }
        },
        api: {
          getPlatforms: () => Promise.resolve(compiler.platforms),
          getAssets: platform => Promise.resolve(Object.entries(compiler.assetsCache[platform] ?? {}).map(([name, asset]) => ({
            name,
            size: asset.size
          }))),
          getCompilationStats: platform => Promise.resolve(compiler.statsCache[platform] ?? null)
        }
      };
    }
  });
  await start();
  compiler.start();
  return {
    stop: async () => {
      reporter.stop();
      await stop();
    }
  };
}
function bindKeypressInput(ctx) {
  if (!process.stdin.setRawMode) {
    ctx.log.warn({
      msg: 'Interactive mode is not supported in this environment'
    });
    return;
  }
  _nodeReadline.default.emitKeypressEvents(process.stdin);
  process.stdin.setRawMode(true);
  process.stdin.on('keypress', (_key, data) => {
    const {
      ctrl,
      name
    } = data;
    if (ctrl === true) {
      switch (name) {
        case 'c':
          process.exit();
          break;
        case 'z':
          process.emit('SIGTSTP', 'SIGTSTP');
          break;
      }
    } else if (name === 'r') {
      ctx.broadcastToMessageClients({
        method: 'reload'
      });
      ctx.log.info({
        msg: 'Reloading app'
      });
    } else if (name === 'd') {
      ctx.broadcastToMessageClients({
        method: 'devMenu'
      });
      ctx.log.info({
        msg: 'Opening developer menu'
      });
    }
  });
}
async function runAdbReverse(ctx, port) {
  const adbPath = process.env.ANDROID_HOME ? `${process.env.ANDROID_HOME}/platform-tools/adb` : 'adb';
  const command = `${adbPath} reverse tcp:${port} tcp:${port}`;
  try {
    await _execa.default.command(command);
    ctx.log.info(`Successfully run: ${command}`);
  } catch (error) {
    // Get just the error message
    const message = error.message.split('error:')[1] || error.message;
    ctx.log.warn(`Failed to run: ${command} - ${message.trim()}`);
  }
}
function parseFileUrl(fileUrl) {
  const {
    pathname: filename,
    searchParams
  } = new _nodeUrl.URL(fileUrl);
  let platform = searchParams.get('platform');
  if (!platform) {
    const [, platformOrName, name] = filename.split('.').reverse();
    if (name !== undefined) {
      platform = platformOrName;
    }
  }
  return {
    filename: filename.replace(/^\//, ''),
    platform: platform || undefined
  };
}
//# sourceMappingURL=start.js.map