"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Compiler = void 0;
var _nodeFs = _interopRequireDefault(require("node:fs"));
var _nodePath = _interopRequireDefault(require("node:path"));
var _memfs = _interopRequireDefault(require("memfs"));
var _mimeTypes = _interopRequireDefault(require("mime-types"));
var _core = _interopRequireDefault(require("@rspack/core"));
var _loadConfig = require("./loadConfig");
var _utils = require("./utils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
class Compiler {
  assetsCache = {};
  statsCache = {};
  resolvers = {};
  isCompilationInProgress = {};
  watchOptions = {};
  watching = null;
  constructor(cliOptions, reporter) {
    this.cliOptions = cliOptions;
    this.reporter = reporter;
    // TODO validate platforms?
    this.platforms = cliOptions.arguments.start.platforms;
  }
  getCompilerForPlatform(platform) {
    if (!this.instance) {
      throw new Error('[Compiler] Compiler not created yet');
    }
    const index = this.platforms.indexOf(platform);
    if (index === -1) {
      throw new Error(`[Compiler] Platform ${platform} not found`);
    }
    const compiler = this.instance.compilers[index];
    if (!compiler) {
      throw new Error(`[Compiler] Compiler for ${platform} not found`);
    }
    return compiler;
  }
  callPendingResolvers(platform, error) {
    this.resolvers[platform]?.forEach(resolver => resolver(error));
    this.resolvers[platform] = [];
  }
  configureCompilerForPlatform(ctx, platform) {
    const internalName = `${platform}-compiler`;
    const platformCompiler = this.getCompilerForPlatform(platform);
    const platformFilesystem = _memfs.default.createFsFromVolume(new _memfs.default.Volume());

    // @ts-expect-error memfs is compatible enough
    platformCompiler.outputFileSystem = platformFilesystem;
    platformCompiler.name = platform;
    platformCompiler.hooks.watchRun.tap(internalName, () => {
      this.isCompilationInProgress[platform] = true;
      ctx.notifyBuildStart(platform);
    });
    platformCompiler.hooks.invalid.tap(internalName, () => {
      this.isCompilationInProgress[platform] = true;
      ctx.notifyBuildStart(platform);
      ctx.broadcastToHmrClients({
        action: 'building'
      }, platform);
    });
    platformCompiler.hooks.done.tap(internalName, stats => {
      this.statsCache[platform] = stats.toJson({
        all: false,
        assets: true,
        children: true,
        timings: true,
        hash: true,
        errors: true,
        warnings: true
      });
      const outputDirectory = stats.compilation.outputOptions.path;
      const assets = this.statsCache[platform].assets;
      this.assetsCache[platform] = assets.reduce((acc, {
        name,
        info,
        size
      }) => {
        const assetPath = _nodePath.default.join(outputDirectory, name);
        const data = platformFilesystem.readFileSync(assetPath);
        const asset = {
          data,
          info,
          size
        };
        return Object.assign(acc, {
          [(0, _utils.adaptFilenameToPlatform)(name)]: asset
        });
      },
      // keep old assets, discard HMR-related ones
      Object.fromEntries(Object.entries(this.assetsCache[platform] ?? {}).filter(([_, asset]) => !asset.info.hotModuleReplacement)));
      this.isCompilationInProgress[platform] = false;
      this.callPendingResolvers(platform);
      ctx.notifyBuildEnd(platform);
      ctx.broadcastToHmrClients({
        action: 'built',
        body: this.getHmrBody(platform)
      }, platform);
    });
  }
  async init(ctx) {
    const webpackEnvOptions = (0, _utils.getEnvOptions)(this.cliOptions);
    const configs = await Promise.all(this.platforms.map(platform => {
      const env = {
        ...webpackEnvOptions,
        platform
      };
      return (0, _loadConfig.loadConfig)(this.cliOptions.config.webpackConfigPath, env);
    }));
    this.instance = _core.default.rspack(configs);
    this.watchOptions = configs[0].watchOptions ?? {};
    this.platforms.forEach(platform => {
      this.configureCompilerForPlatform(ctx, platform);
    });
    this.instance.hooks.done.tap('Compiler', multiStats => {
      const issuer = 'DevServer';
      const timestamp = Date.now();
      const {
        children = [],
        errors = [],
        warnings = []
      } = multiStats.toJson({
        preset: 'errors-warnings',
        timings: true
      });
      if (errors.length) {
        const message = ['Bundle built with errors'];
        this.reporter.process({
          type: 'error',
          issuer,
          timestamp,
          message
        });
        children.forEach(stats => {
          stats.errors?.forEach(error => {
            const message = [`Error in "${error.moduleName}":\n${error.formatted}`];
            this.reporter.process({
              type: 'error',
              issuer,
              timestamp,
              message
            });
          });
        });
      } else if (warnings?.length) {
        const message = ['Bundle built with warnings'];
        this.reporter.process({
          type: 'warn',
          issuer,
          timestamp,
          message
        });
        children.forEach(stats => {
          stats.warnings?.forEach(warning => {
            const message = [`Warning in "${warning.moduleName}": ${warning.message}`];
            this.reporter.process({
              type: 'warn',
              issuer,
              timestamp,
              message
            });
          });
        });
      } else {
        const message = ['Bundle built', Object.fromEntries(children.map(({
          name,
          time
        }) => [name, time]))];
        this.reporter.process({
          type: 'success',
          issuer,
          timestamp,
          message
        });
      }
      this.reporter.flush();
      this.reporter.stop();
    });
  }
  start() {
    this.reporter.process({
      type: 'info',
      issuer: 'DevServer',
      timestamp: Date.now(),
      message: ['Starting build for platforms:', this.platforms.join(', ')]
    });
    this.watching = this.instance.watch(this.watchOptions, error => {
      if (!error) return;
      this.platforms.forEach(platform => {
        this.callPendingResolvers(platform, error);
      });
    });
  }
  async getAsset(filename, platform) {
    // Return file from assetsCache if exists
    const fileFromCache = this.assetsCache[platform]?.[filename];
    if (fileFromCache) {
      return fileFromCache;
    }
    if (!this.isCompilationInProgress[platform]) {
      return Promise.reject(new Error(`File ${filename} for ${platform} not found in compilation assets (no compilation in progress)`));
    }
    return await new Promise((resolve, reject) => {
      // Add new resolver to be executed when compilation is finished
      this.resolvers[platform] = (this.resolvers[platform] ?? []).concat(error => {
        if (error) {
          reject(error);
        } else {
          const fileFromCache = this.assetsCache[platform]?.[filename];
          if (fileFromCache) {
            resolve(fileFromCache);
          } else {
            reject(new Error(`File ${filename} for ${platform} not found in compilation assets`));
          }
        }
      });
    });
  }
  async getSource(filename, platform) {
    /**
     * TODO refactor this part
     *
     * This code makes an assumption that filename ends with .bundle
     * but this can be changed by the user, so is prone to breaking
     * In reality, it's not that big a deal. This part is within a dev server
     * so we might override & enforce the format for the purpose of development
     */
    if (/\.bundle/.test(filename) && platform) {
      return (await this.getAsset(filename, platform)).data;
    }
    return _nodeFs.default.promises.readFile(_nodePath.default.join(this.cliOptions.config.root, filename), 'utf8');
  }
  async getSourceMap(filename, platform) {
    /**
     * Inside dev server we can control the naming of sourcemaps
     * so there is no need to look it up, we can just assume default naming scheme
     */
    const sourceMapFilename = filename + '.map';
    try {
      const sourceMap = await this.getAsset(sourceMapFilename, platform);
      return sourceMap.data;
    } catch {
      throw new Error(`Source map for ${filename} for ${platform} is missing`);
    }
  }
  getMimeType(filename) {
    /**
     * TODO potentially refactor
     *
     * same as in getSource, this part is prone to breaking
     * if the user changes the filename format
     */
    if (filename.endsWith('.bundle')) {
      return 'text/javascript';
    }
    return _mimeTypes.default.lookup(filename) || 'text/plain';
  }
  getHmrBody(platform) {
    const stats = this.statsCache[platform];
    if (!stats) {
      return null;
    }
    return {
      name: stats.name ?? '',
      time: stats.time ?? 0,
      hash: stats.hash ?? '',
      warnings: stats.warnings || [],
      errors: stats.errors || []
    };
  }
}
exports.Compiler = Compiler;
//# sourceMappingURL=Compiler.js.map