"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LoggerPlugin = void 0;
var _env = require("../../env");
var _logging = require("../../logging");
/**
 * {@link LoggerPlugin} configuration options.
 */

/**
 * Logger plugin that handles all logging coming from the Webpack ecosystem,
 * including debug logs from other plugins and resolvers.
 *
 * @category Webpack Plugin
 */
class LoggerPlugin {
  static SUPPORTED_TYPES = ['debug', 'info', 'warn', 'error', 'success'];

  /** {@link Reporter} instance used to actually writing logs to terminal/file. */

  /**
   * Constructs new `LoggerPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
    if (this.config.output === undefined) {
      this.config.output = {
        console: true
      };
    }
    const reporters = [];
    if (this.config.output.console) {
      reporters.push(new _logging.ConsoleReporter({
        level: (0, _env.isVerbose)() ? 'verbose' : 'normal'
      }));
    }
    if (this.config.output.file) {
      reporters.push(new _logging.FileReporter({
        filename: this.config.output.file
      }));
    }
    this.reporter = (0, _logging.composeReporters)(reporters);
  }

  /**
   * Create log entry from Webpack log message from {@link WebpackLogger}.
   *
   * @param issuer Issuer of the message.
   * @param type Type of the message.
   * @param args The body of the message.
   * @param timestamp Timestamp when the message was recorder.
   * @returns Log entry object or undefined when if message is invalid.
   */
  createEntry(issuer, type, args, timestamp) {
    if (LoggerPlugin.SUPPORTED_TYPES.includes(type)) {
      return {
        timestamp: timestamp ?? Date.now(),
        issuer: issuer.includes('reactNativeAssetsLoader') ? 'reactNativeAssetsLoader' : issuer,
        type: type,
        message: args
      };
    }
    return undefined;
  }

  /**
   * Process log entry and pass it to {@link reporter} instance.
   *
   * @param entry Log entry to process
   */
  processEntry(entry) {
    if (!this.config.output?.console && !this.config.output?.file && !this.config.output?.listener) {
      return;
    }
    this.reporter.process(entry);
    if (this.config.output.listener) {
      this.config.output.listener(entry);
    }
  }

  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */
  apply(compiler) {
    // Make sure webpack-cli doesn't print stats by default.
    if (compiler.options.stats === undefined) {
      compiler.options.stats = 'none';
    }
    compiler.hooks.infrastructureLog.tap('LoggerPlugin', (issuer, type, args) => {
      const entry = this.createEntry(issuer, type, args);
      if (entry) {
        this.processEntry(entry);
      }
      return true;
    });
    compiler.hooks.thisCompilation.tap('LoggerPlugin', compilation => {
      compilation.hooks.log.intercept({
        call: (issuer, {
          time,
          type,
          args
        }) => {
          const entry = this.createEntry(issuer, type, args, time);
          if (entry) {
            this.processEntry(entry);
          }
        }
      });
    });
    if (!this.config.devServerEnabled) {
      compiler.hooks.done.tap('LoggerPlugin', stats => {
        const statsEntry = this.createEntry('LoggerPlugin', 'info', [stats.toString({
          preset: 'normal',
          colors: true
        })]);
        if (statsEntry) {
          this.processEntry(statsEntry);
        }
        this.reporter.flush();
        this.reporter.stop();
      });
    }
    process.on('uncaughtException', error => {
      const errorEntry = this.createEntry('LoggerPlugin', 'error', [error]);
      if (errorEntry) {
        this.processEntry(errorEntry);
      }
      this.reporter.flush();
      this.reporter.stop();
    });
    process.on('exit', () => {
      this.reporter.flush();
      this.reporter.stop();
    });
  }
}
exports.LoggerPlugin = LoggerPlugin;
//# sourceMappingURL=LoggerPlugin.js.map