"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RepackTargetPlugin = void 0;
var _nodePath = _interopRequireDefault(require("node:path"));
var _core = _interopRequireDefault(require("@rspack/core"));
var _RepackLoadScriptRuntimeModule = require("./runtime/RepackLoadScriptRuntimeModule");
var _RepackInitRuntimeModule = require("./runtime/RepackInitRuntimeModule");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/**
 * {@link RepackTargetPlugin} configuration options.
 */

/**
 * Plugin for tweaking the JavaScript runtime code to account for React Native environment.
 *
 * Globally available APIs differ with React Native and other target's like Web, so there are some
 * tweaks necessary to make the final bundle runnable inside React Native's JavaScript VM.
 *
 * @category Webpack Plugin
 */
class RepackTargetPlugin {
  /**
   * Constructs new `RepackTargetPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
  }
  getReactNativePath(candidate) {
    let reactNativePath;
    if (typeof candidate === 'string') {
      reactNativePath = candidate;
    }
    if (typeof candidate === 'object') {
      const candidates = candidate.filter(Boolean);
      reactNativePath = candidates[0];
    }
    if (!reactNativePath) {
      reactNativePath = require.resolve('react-native');
    }
    return _nodePath.default.dirname(reactNativePath);
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */
  apply(compiler) {
    const globalObject = 'self';
    compiler.options.target = false;
    compiler.options.output.chunkLoading = 'jsonp';
    compiler.options.output.chunkFormat = 'array-push';
    compiler.options.output.globalObject = globalObject;
    const reactNativePath = this.getReactNativePath(compiler.options.resolve.alias?.['react-native']);
    const getReactNativePolyfills = require(_nodePath.default.join(reactNativePath, 'rn-get-polyfills.js'));
    const entries = [...getReactNativePolyfills(), this.config?.initializeCoreLocation || _nodePath.default.join(reactNativePath, 'Libraries/Core/InitializeCore.js'), require.resolve('../../../modules/configurePublicPath')];

    // Add React-Native entries
    for (const entry of entries) {
      new _core.default.EntryPlugin(compiler.context, entry, {
        name: undefined
      }).apply(compiler);
    }

    // Normalize global object.
    new _core.default.BannerPlugin({
      raw: true,
      entryOnly: true,
      banner: _core.default.Template.asString([`/******/ var ${globalObject} = ${globalObject} || this || new Function("return this")() || ({}); // repackGlobal'`, '/******/'])
    }).apply(compiler);

    // Replace React Native's HMRClient.js with custom Webpack-powered DevServerClient.
    new _core.default.NormalModuleReplacementPlugin(/react-native.*?([/\\]+)Libraries[/\\]Utilities[/\\]HMRClient\.js$/, require.resolve('../../../modules/DevServerClient')).apply(compiler);

    // ReactNativeTypes.js is flow type only module
    new _core.default.NormalModuleReplacementPlugin(/react-native.*?([/\\]+)Libraries[/\\]Renderer[/\\]shims[/\\]ReactNativeTypes\.js$/, require.resolve('../../../modules/EmptyModule')).apply(compiler);
    compiler.hooks.thisCompilation.tap('RepackTargetPlugin', compilation => {
      compilation.hooks.runtimeModule.tap('RepackTargetPlugin', (module, chunk) => {
        // TODO determine if we need limit it to just the main chunk
        /**
         * We inject RePack's runtime modules only when load_script module is present.
         * This module is injected when:
         * 1. HMR is enabled
         * 2. Dynamic import is used anywhere in the project
         */
        if (module.name === 'load_script') {
          const loadScriptRuntimeModule = (0, _RepackLoadScriptRuntimeModule.generateLoadScriptRuntimeModule)(chunk.id);
          const initRuntimeModule = (0, _RepackInitRuntimeModule.generateRepackInitRuntimeModule)({
            chunkId: chunk.id,
            chunkLoadingGlobal: compiler.options.output.chunkLoadingGlobal,
            globalObject: globalObject,
            hmrEnabled: this.config?.hmr
          });

          // combine both runtime modules
          const repackRuntimeModule = Buffer.from(`${loadScriptRuntimeModule}\n${initRuntimeModule}`, 'utf-8');

          // inject runtime module
          module.source.source = repackRuntimeModule;
        }

        // Remove CSS runtime modules
        if (module.name === 'css_loading' || module.name === 'get css chunk filename') {
          module.source.source = Buffer.from(`// noop`, 'utf-8');
        }
      });
    });
  }
}
exports.RepackTargetPlugin = RepackTargetPlugin;
//# sourceMappingURL=RepackTargetPlugin.js.map